@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 //! [`LocalYamlRegistry`]: struct.LocalYamlRegistry.html
 //! [`RwRegistry`]: ../../trait.RwRegistry.html
 
+use std::convert::TryFrom;
 use std::fs::File;
 use std::io::Write;
 use std::path::PathBuf;
@@ -27,10 +28,14 @@ use std::sync::{Arc, Mutex};
 use std::time::SystemTime;
 
 use crate::registry::{
-    validate_nodes, MetadataPredicate, Node, NodeIter, RegistryError, RegistryReader,
-    RegistryWriter, RwRegistry,
+    error::InvalidNodeError, validate_nodes, MetadataPredicate, Node, NodeIter, RegistryError,
+    RegistryReader, RegistryWriter, RwRegistry,
 };
 
+use crate::error::{InternalError, InvalidStateError};
+
+use super::YamlNode;
+
 /// A local, read/write registry.
 ///
 /// The `LocalYamlRegistry` provides access to and modification of a local registry YAML file. The
@@ -72,7 +77,11 @@ impl LocalYamlRegistry {
         Ok(self
             .internal
             .lock()
-            .map_err(|_| RegistryError::general_error("YAML registry's internal lock poisoned"))?
+            .map_err(|_| {
+                RegistryError::InternalError(InternalError::with_message(
+                    "YAML registry's internal lock poisoned".into(),
+                ))
+            })?
             .get_nodes())
     }
 
@@ -80,13 +89,17 @@ impl LocalYamlRegistry {
     pub(super) fn write_nodes(&self, nodes: Vec<Node>) -> Result<(), RegistryError> {
         self.internal
             .lock()
-            .map_err(|_| RegistryError::general_error("YAML registry's internal lock poisoned"))?
+            .map_err(|_| {
+                RegistryError::InternalError(InternalError::with_message(
+                    "YAML registry's internal lock poisoned".into(),
+                ))
+            })?
             .write_nodes(nodes)
     }
 }
 
 impl RegistryReader for LocalYamlRegistry {
-    fn fetch_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
+    fn get_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
         Ok(self
             .get_nodes()?
             .iter()
@@ -110,17 +123,38 @@ impl RegistryReader for LocalYamlRegistry {
             .filter(move |node| predicates.iter().all(|predicate| predicate.apply(node)))
             .count() as u32)
     }
+
+    fn has_node(&self, identity: &str) -> Result<bool, RegistryError> {
+        Ok(self
+            .get_nodes()?
+            .iter()
+            .any(|node| node.identity == identity))
+    }
 }
 
 impl RegistryWriter for LocalYamlRegistry {
-    fn insert_node(&self, node: Node) -> Result<(), RegistryError> {
+    fn add_node(&self, node: Node) -> Result<(), RegistryError> {
         let mut nodes = self.get_nodes()?;
-        // If a node with the same identity already exists, remove it
-        nodes.retain(|existing_node| existing_node.identity != node.identity);
         nodes.push(node);
         self.write_nodes(nodes)
     }
 
+    fn update_node(&self, node: Node) -> Result<(), RegistryError> {
+        let mut nodes = self.get_nodes()?;
+        if nodes.iter().any(|n| n.identity == node.identity) {
+            nodes.retain(|existing_node| existing_node.identity != node.identity);
+            nodes.push(node);
+            self.write_nodes(nodes)
+        } else {
+            Err(RegistryError::InvalidStateError(
+                InvalidStateError::with_message(format!(
+                    "Node does not exist in the registry: {}",
+                    node.identity
+                )),
+            ))
+        }
+    }
+
     fn delete_node(&self, identity: &str) -> Result<Option<Node>, RegistryError> {
         let mut nodes = self.get_nodes()?;
         let mut index = None;
@@ -183,10 +217,10 @@ impl Internal {
         let file_read_result = std::fs::metadata(&self.file_path)
             .and_then(|metadata| metadata.modified())
             .map_err(|err| {
-                RegistryError::general_error_with_source(
-                    "Failed to read YAML registry file's last modification time",
+                RegistryError::InternalError(InternalError::from_source_with_message(
                     Box::new(err),
-                )
+                    "Failed to read YAML registry file's last modification time".into(),
+                ))
             })
             .and_then(|last_modified| {
                 if last_modified > self.last_read {
@@ -211,19 +245,33 @@ impl Internal {
     /// Read the backing file, verify that it's valid, and cache its contents.
     fn read_nodes(&mut self) -> Result<(), RegistryError> {
         let file = File::open(&self.file_path).map_err(|err| {
-            RegistryError::general_error_with_source(
-                "Failed to open YAML registry file",
+            RegistryError::InternalError(InternalError::from_source_with_message(
                 Box::new(err),
-            )
-        })?;
-        let nodes: Vec<Node> = serde_yaml::from_reader(&file).map_err(|err| {
-            RegistryError::general_error_with_source(
-                "Failed to read YAML registry file",
-                Box::new(err),
-            )
+                "Failed to open YAML registry file".into(),
+            ))
         })?;
 
-        validate_nodes(&nodes)?;
+        let yaml_nodes: Vec<YamlNode> = serde_yaml::from_reader(&file).map_err(|err| {
+            RegistryError::InternalError(InternalError::from_source_with_message(
+                Box::new(err),
+                "Failed to read YAML registry file".into(),
+            ))
+        })?;
+
+        let nodes: Vec<Node> = yaml_nodes
+            .into_iter()
+            .map(Node::try_from)
+            .collect::<Result<Vec<Node>, InvalidNodeError>>()
+            .map_err(|err| {
+                RegistryError::InvalidStateError(InvalidStateError::with_message(format!(
+                    "Unable to get node list: {}",
+                    err
+                )))
+            })?;
+
+        validate_nodes(&nodes).map_err(|err| {
+            RegistryError::InvalidStateError(InvalidStateError::with_message(err.to_string()))
+        })?;
 
         self.cached_nodes = nodes;
         self.last_read = SystemTime::now();
@@ -234,30 +282,42 @@ impl Internal {
     /// Verify that the given nodes represent a valid registry, write them to the backing file, and
     /// update the in-memory cache.
     fn write_nodes(&mut self, nodes: Vec<Node>) -> Result<(), RegistryError> {
-        validate_nodes(&nodes)?;
+        validate_nodes(&nodes).map_err(|err| {
+            RegistryError::InvalidStateError(InvalidStateError::with_message(err.to_string()))
+        })?;
 
-        let output = serde_yaml::to_vec(&nodes).map_err(|err| {
-            RegistryError::general_error_with_source("Failed to write nodes to YAML", Box::new(err))
+        let yaml_nodes: Vec<YamlNode> = nodes
+            .iter()
+            .map(|node| YamlNode::from(node.clone()))
+            .collect::<Vec<YamlNode>>();
+
+        let output = serde_yaml::to_vec(&yaml_nodes).map_err(|err| {
+            RegistryError::InternalError(InternalError::from_source_with_message(
+                Box::new(err),
+                "Failed to write nodes to YAML".into(),
+            ))
         })?;
 
         let mut file = File::create(&self.file_path).map_err(|err| {
-            RegistryError::general_error_with_source(
-                &format!("Failed to open YAML registry file '{}'", self.file_path),
+            RegistryError::InternalError(InternalError::from_source_with_message(
                 Box::new(err),
-            )
+                format!("Failed to open YAML registry file '{}'", self.file_path),
+            ))
         })?;
+
         file.write_all(&output).map_err(|err| {
-            RegistryError::general_error_with_source(
-                &format!("Failed to write to YAML registry file '{}'", self.file_path),
+            RegistryError::InternalError(InternalError::from_source_with_message(
                 Box::new(err),
-            )
+                format!("Failed to write to YAML registry file '{}'", self.file_path),
+            ))
         })?;
+
         // Append newline to file
         writeln!(file).map_err(|err| {
-            RegistryError::general_error_with_source(
-                &format!("Failed to write to YAML registry file '{}'", self.file_path),
+            RegistryError::InternalError(InternalError::from_source_with_message(
                 Box::new(err),
-            )
+                format!("Failed to write to YAML registry file '{}'", self.file_path),
+            ))
         })?;
 
         self.cached_nodes = nodes;
@@ -275,8 +335,6 @@ mod test {
 
     use tempdir::TempDir;
 
-    use crate::registry::InvalidNodeError;
-
     ///
     /// Verifies that reading from a YAML file that contains two nodes with the same identity
     /// returns InvalidNodeError::DuplicateIdentity.
@@ -301,11 +359,9 @@ mod test {
         let result = LocalYamlRegistry::new(&path);
         match result {
             Ok(_) => panic!("Two nodes with same identity in YAML file. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::DuplicateIdentity(id))) => {
-                assert_eq!(id, node1.identity)
-            }
+            Err(RegistryError::InvalidStateError(_)) => (),
             Err(err) => panic!(
-                "Should have gotten InvalidNodeError::DuplicateIdentity but got {}",
+                "Should have gotten RegistryError::InvalidStateError but got {}",
                 err
             ),
         }
@@ -335,11 +391,9 @@ mod test {
         let result = LocalYamlRegistry::new(&path);
         match result {
             Ok(_) => panic!("Two nodes with same endpoint in YAML file. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::DuplicateEndpoint(endpoint))) => {
-                assert!(node1.endpoints.contains(&endpoint))
-            }
+            Err(RegistryError::InvalidStateError(_)) => (),
             Err(err) => panic!(
-                "Should have gotten InvalidNodeError::DuplicateEndpoint but got {}",
+                "Should have gotten RegistryError::InvalidStateError but got {}",
                 err
             ),
         }
@@ -368,9 +422,9 @@ mod test {
         let result = LocalYamlRegistry::new(&path);
         match result {
             Ok(_) => panic!("Node with empty identity in YAML file. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::EmptyIdentity)) => {}
+            Err(RegistryError::InvalidStateError(_)) => (),
             Err(err) => panic!(
-                "Should have gotten InvalidNodeError::EmptyIdentity but got {}",
+                "Should have gotten RegistryError::InvalidStateError but got {}",
                 err
             ),
         }
@@ -399,9 +453,9 @@ mod test {
         let result = LocalYamlRegistry::new(&path);
         match result {
             Ok(_) => panic!("Node with empty endpoint in YAML file. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::EmptyEndpoint)) => {}
+            Err(RegistryError::InvalidStateError(_)) => (),
             Err(err) => panic!(
-                "Should have gotten InvalidNodeError::EmptyEndpoint but got {}",
+                "Should have gotten RegistryError::InvalidStateError but got {}",
                 err
             ),
         }
@@ -430,9 +484,9 @@ mod test {
         let result = LocalYamlRegistry::new(&path);
         match result {
             Ok(_) => panic!("Node with empty display_name in YAML file. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::EmptyDisplayName)) => {}
+            Err(RegistryError::InvalidStateError(_)) => (),
             Err(err) => panic!(
-                "Should have gotten InvalidNodeError::EmptyDisplayName but got {}",
+                "Should have gotten RegistryError::InvalidStateError but got {}",
                 err
             ),
         }
@@ -461,9 +515,9 @@ mod test {
         let result = LocalYamlRegistry::new(&path);
         match result {
             Ok(_) => panic!("Node with empty key in YAML file. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::EmptyKey)) => {}
+            Err(RegistryError::InvalidStateError(_)) => (),
             Err(err) => panic!(
-                "Should have gotten InvalidNodeError::EmptyKey but got {}",
+                "Should have gotten RegistryError::InvalidStateError but got {}",
                 err
             ),
         }
@@ -492,9 +546,9 @@ mod test {
         let result = LocalYamlRegistry::new(&path);
         match result {
             Ok(_) => panic!("Node with no endpoint in YAML file. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::MissingEndpoints)) => {}
+            Err(RegistryError::InvalidStateError(_)) => (),
             Err(err) => panic!(
-                "Should have gotten InvalidNodeError::MissingEndpoints but got {}",
+                "Should have gotten RegistryError::InvalidStateError but got {}",
                 err
             ),
         }
@@ -523,20 +577,20 @@ mod test {
         let result = LocalYamlRegistry::new(&path);
         match result {
             Ok(_) => panic!("Node with no keys in YAML file. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::MissingKeys)) => {}
+            Err(RegistryError::InvalidStateError(_)) => (),
             Err(err) => panic!(
-                "Should have gotten InvalidNodeError::MissingKeys but got {}",
+                "Should have gotten RegistryError::InvalidStateError but got {}",
                 err
             ),
         }
     }
 
     ///
-    /// Verifies that fetch_node with a valid identity, returns the correct node.
+    /// Verifies that get_node with a valid identity, returns the correct node.
     ///
     #[test]
-    fn test_fetch_node_ok() {
-        let temp_dir = TempDir::new("test_fetch_node_ok").expect("Failed to create temp dir");
+    fn test_get_node_ok() {
+        let temp_dir = TempDir::new("test_get_node_ok").expect("Failed to create temp dir");
         let path = temp_dir
             .path()
             .join("registry.yaml")
@@ -549,19 +603,18 @@ mod test {
         let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
 
         let node = registry
-            .fetch_node(&get_node_1().identity)
+            .get_node(&get_node_1().identity)
             .expect("Failed to fetch node")
             .expect("Node not found");
         assert_eq!(node, get_node_1());
     }
 
     ///
-    /// Verifies that fetch_node with an invalid identity returns Ok(None)
+    /// Verifies that get_node with an invalid identity returns Ok(None)
     ///
     #[test]
-    fn test_fetch_node_not_found() {
-        let temp_dir =
-            TempDir::new("test_fetch_node_not_found").expect("Failed to create temp dir");
+    fn test_get_node_not_found() {
+        let temp_dir = TempDir::new("test_get_node_not_found").expect("Failed to create temp dir");
         let path = temp_dir
             .path()
             .join("registry.yaml")
@@ -573,13 +626,38 @@ mod test {
 
         let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
 
-        let result = registry.fetch_node("NodeNotInRegistry");
+        let result = registry.get_node("NodeNotInRegistry");
         match result {
             Ok(None) => {}
             res => panic!("Should have gotten Ok(None) but got {:?}", res),
         }
     }
 
+    ///
+    /// Verifies that `has_node` properly determines if a node exists in the registry.
+    ///
+    #[test]
+    fn test_has_node() {
+        let temp_dir = TempDir::new("test_has_node").expect("Failed to create temp dir");
+        let path = temp_dir
+            .path()
+            .join("registry.yaml")
+            .to_str()
+            .expect("Failed to get path")
+            .to_string();
+
+        write_to_file(&vec![get_node_1()], &path);
+
+        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
+
+        assert!(registry
+            .has_node(&get_node_1().identity)
+            .expect("Failed to check if node1 exists"));
+        assert!(!registry
+            .has_node(&get_node_2().identity)
+            .expect("Failed to check if node2 exists"));
+    }
+
     ///
     /// Verifies that list_nodes returns a list of nodes.
     ///
@@ -733,11 +811,48 @@ mod test {
     }
 
     ///
-    /// Verifies that insert_node successfully adds a new node to the yaml file.
+    /// Verifies that add_node returns InvalidNodeError::DuplicateEndpoint when a node
+    /// with the same endpoint already exists in the yaml file.
     ///
     #[test]
-    fn test_add_node_ok() {
-        let temp_dir = TempDir::new("test_add_node_ok").expect("Failed to create temp dir");
+    fn test_add_node_duplicate_endpoint_error() {
+        let temp_dir = TempDir::new("test_xv_node_duplicate_endpoint_error")
+            .expect("Failed to create temp dir");
+        let path = temp_dir
+            .path()
+            .join("registry.yaml")
+            .to_str()
+            .expect("Failed to get path")
+            .to_string();
+
+        let node1 = get_node_1();
+
+        write_to_file(&vec![node1.clone()], &path);
+
+        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
+
+        let mut node = get_node_2();
+        node.endpoints = node1.endpoints.clone();
+        let result = registry.add_node(node);
+
+        match result {
+            Ok(_) => panic!("Node with endpoint already exists. Error should be returned"),
+            Err(RegistryError::InvalidStateError(_)) => {}
+            Err(err) => panic!(
+                "Should have gotten RegistryError::InvalidStateError but got {}",
+                err
+            ),
+        }
+    }
+
+    ///
+    /// Verifies that add_node returns InvalidNodeError::EmptyIdentity when a node with
+    /// an empty string as its identity is added to the registry.
+    ///
+    #[test]
+    fn test_add_node_empty_identity_error() {
+        let temp_dir =
+            TempDir::new("test_add_node_empty_identity_error").expect("Failed to create temp dir");
         let path = temp_dir
             .path()
             .join("registry.yaml")
@@ -749,22 +864,187 @@ mod test {
 
         let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
 
-        let node = get_node_1();
+        let mut node = get_node_1();
+        node.identity = "".to_string();
+        let result = registry.add_node(node);
 
-        registry
-            .insert_node(node.clone())
-            .expect("Failed to insert node");
-
-        let nodes = registry
-            .list_nodes(&[])
-            .expect("Failed to retrieve nodes")
-            .collect::<Vec<_>>();
-
-        assert_eq!(nodes, vec![node]);
+        match result {
+            Ok(_) => panic!("Node identity is empty. Error should be returned"),
+            Err(RegistryError::InvalidStateError(_)) => {}
+            Err(err) => panic!(
+                "Should have gotten RegistryError::InvalidStateError but got {}",
+                err
+            ),
+        }
     }
 
     ///
-    /// Verifies that insert_node successfully updates an existing node in the yaml file.
+    /// Verifies that add_node returns InvalidNodeError::EmptyEndpoint when a node with
+    /// an empty string in its endpoints is added to the registry.
+    ///
+    #[test]
+    fn test_add_node_empty_endpoint_error() {
+        let temp_dir =
+            TempDir::new("test_add_node_empty_endpoint_error").expect("Failed to create temp dir");
+        let path = temp_dir
+            .path()
+            .join("registry.yaml")
+            .to_str()
+            .expect("Failed to get path")
+            .to_string();
+
+        write_to_file(&vec![], &path);
+
+        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
+
+        let mut node = get_node_1();
+        node.endpoints = vec!["".to_string()];
+        let result = registry.add_node(node);
+
+        match result {
+            Ok(_) => panic!("Node endpoint is empty. Error should be returned"),
+            Err(RegistryError::InvalidStateError(_)) => {}
+            Err(err) => panic!(
+                "Should have gotten RegistryError::InvalidStateError but got {}",
+                err
+            ),
+        }
+    }
+
+    ///
+    /// Verifies that add_node returns InvalidNodeError::EmptyDisplayName when a node
+    /// with an empty string as its display_name is added to the registry.
+    ///
+    #[test]
+    fn test_add_node_empty_display_name_error() {
+        let temp_dir = TempDir::new("test_add_node_missing_endpoints_error")
+            .expect("Failed to create temp dir");
+        let path = temp_dir
+            .path()
+            .join("registry.yaml")
+            .to_str()
+            .expect("Failed to get path")
+            .to_string();
+
+        write_to_file(&vec![], &path);
+
+        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
+
+        let mut node = get_node_1();
+        node.display_name = "".to_string();
+        let result = registry.add_node(node);
+
+        match result {
+            Ok(_) => panic!("Node display_name is empty. Error should be returned"),
+            Err(RegistryError::InvalidStateError(_)) => {}
+            Err(err) => panic!(
+                "Should have gotten RegistryError::InvalidStateError but got {}",
+                err
+            ),
+        }
+    }
+
+    ///
+    /// Verifies that add_node returns InvalidNodeError::EmptyKey when a node with
+    /// an empty string in its keys is added to the registry.
+    ///
+    #[test]
+    fn test_add_node_empty_key_error() {
+        let temp_dir = TempDir::new("test_add_node_missing_endpoints_error")
+            .expect("Failed to create temp dir");
+        let path = temp_dir
+            .path()
+            .join("registry.yaml")
+            .to_str()
+            .expect("Failed to get path")
+            .to_string();
+
+        write_to_file(&vec![], &path);
+
+        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
+
+        let mut node = get_node_1();
+        node.keys = vec!["".to_string()];
+        let result = registry.add_node(node);
+
+        match result {
+            Ok(_) => panic!("Node key is empty. Error should be returned"),
+            Err(RegistryError::InvalidStateError(_)) => {}
+            Err(err) => panic!(
+                "Should have gotten RegistryError::InvalidStateError but got {}",
+                err
+            ),
+        }
+    }
+
+    ///
+    /// Verifies that add_node returns InvalidNodeError::MissingEndpoints when a node with no
+    /// endpoints is added to the registry.
+    ///
+    #[test]
+    fn test_add_node_missing_endpoints_error() {
+        let temp_dir = TempDir::new("test_add_node_missing_endpoints_error")
+            .expect("Failed to create temp dir");
+        let path = temp_dir
+            .path()
+            .join("registry.yaml")
+            .to_str()
+            .expect("Failed to get path")
+            .to_string();
+
+        write_to_file(&vec![], &path);
+
+        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
+
+        let mut node = get_node_1();
+        node.endpoints = vec![];
+        let result = registry.add_node(node);
+
+        match result {
+            Ok(_) => panic!("Node endpoints is empty. Error should be returned"),
+            Err(RegistryError::InvalidStateError(_)) => {}
+            Err(err) => panic!(
+                "Should have gotten RegistryError::InvalidStateError but got {}",
+                err
+            ),
+        }
+    }
+
+    ///
+    /// Verifies that add_node returns InvalidNodeError::MissingKeys when a node with no
+    /// keys is added to the registry.
+    ///
+    #[test]
+    fn test_add_node_missing_keys_error() {
+        let temp_dir = TempDir::new("test_add_node_missing_endpoints_error")
+            .expect("Failed to create temp dir");
+        let path = temp_dir
+            .path()
+            .join("registry.yaml")
+            .to_str()
+            .expect("Failed to get path")
+            .to_string();
+
+        write_to_file(&vec![], &path);
+
+        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
+
+        let mut node = get_node_1();
+        node.keys = vec![];
+        let result = registry.add_node(node);
+
+        match result {
+            Ok(_) => panic!("Node keys is empty. Error should be returned"),
+            Err(RegistryError::InvalidStateError(_)) => {}
+            Err(err) => panic!(
+                "Should have gotten RegistryError::InvalidStateError but got {}",
+                err
+            ),
+        }
+    }
+
+    ///
+    /// Verifies that update_node successfully updates an existing node in the yaml file.
     ///
     #[test]
     fn test_update_node_ok() {
@@ -784,8 +1064,9 @@ mod test {
         node.metadata
             .insert("location".to_string(), "Minneapolis".to_string());
 
+        #[allow(deprecated)]
         registry
-            .insert_node(node.clone())
+            .update_node(node.clone())
             .expect("Failed to insert node");
 
         let nodes = registry
@@ -797,12 +1078,12 @@ mod test {
     }
 
     ///
-    /// Verifies that insert_node returns InvalidNodeError::DuplicateEndpoint when a node
+    /// Verifies that update_node returns InvalidNodeError::DuplicateEndpoint when a node
     /// with the same endpoint already exists in the yaml file.
     ///
     #[test]
-    fn test_insert_node_duplicate_endpoint_error() {
-        let temp_dir = TempDir::new("test_insert_node_duplicate_endpoint_error")
+    fn test_update_node_duplicate_endpoint_error() {
+        let temp_dir = TempDir::new("test_xv_node_duplicate_endpoint_error")
             .expect("Failed to create temp dir");
         let path = temp_dir
             .path()
@@ -812,220 +1093,21 @@ mod test {
             .to_string();
 
         let node1 = get_node_1();
+        let node2 = get_node_2();
 
-        write_to_file(&vec![node1.clone()], &path);
+        write_to_file(&vec![node1.clone(), node2.clone()], &path);
 
         let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
 
         let mut node = get_node_2();
         node.endpoints = node1.endpoints.clone();
-        let result = registry.insert_node(node);
+        let result = registry.update_node(node);
 
         match result {
             Ok(_) => panic!("Node with endpoint already exists. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::DuplicateEndpoint(endpoint))) => {
-                assert!(node1.endpoints.contains(&endpoint))
-            }
+            Err(RegistryError::InvalidStateError(_)) => {}
             Err(err) => panic!(
-                "Should have gotten InvalidNodeError::DuplicateEndpoint but got {}",
-                err
-            ),
-        }
-    }
-
-    ///
-    /// Verifies that insert_node returns InvalidNodeError::EmptyIdentity when a node with
-    /// an empty string as its identity is added to the registry.
-    ///
-    #[test]
-    fn test_insert_node_empty_identity_error() {
-        let temp_dir = TempDir::new("test_insert_node_empty_identity_error")
-            .expect("Failed to create temp dir");
-        let path = temp_dir
-            .path()
-            .join("registry.yaml")
-            .to_str()
-            .expect("Failed to get path")
-            .to_string();
-
-        write_to_file(&vec![], &path);
-
-        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
-
-        let mut node = get_node_1();
-        node.identity = "".to_string();
-        let result = registry.insert_node(node);
-
-        match result {
-            Ok(_) => panic!("Node identity is empty. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::EmptyIdentity)) => {}
-            Err(err) => panic!(
-                "Should have gotten InvalidNodeError::EmptyIdentity but got {}",
-                err
-            ),
-        }
-    }
-
-    ///
-    /// Verifies that insert_node returns InvalidNodeError::EmptyEndpoint when a node with
-    /// an empty string in its endpoints is added to the registry.
-    ///
-    #[test]
-    fn test_insert_node_empty_endpoint_error() {
-        let temp_dir = TempDir::new("test_insert_node_empty_endpoint_error")
-            .expect("Failed to create temp dir");
-        let path = temp_dir
-            .path()
-            .join("registry.yaml")
-            .to_str()
-            .expect("Failed to get path")
-            .to_string();
-
-        write_to_file(&vec![], &path);
-
-        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
-
-        let mut node = get_node_1();
-        node.endpoints = vec!["".to_string()];
-        let result = registry.insert_node(node);
-
-        match result {
-            Ok(_) => panic!("Node endpoint is empty. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::EmptyEndpoint)) => {}
-            Err(err) => panic!(
-                "Should have gotten InvalidNodeError::EmptyEndpoint but got {}",
-                err
-            ),
-        }
-    }
-
-    ///
-    /// Verifies that insert_node returns InvalidNodeError::EmptyDisplayName when a node
-    /// with an empty string as its display_name is added to the registry.
-    ///
-    #[test]
-    fn test_insert_node_empty_display_name_error() {
-        let temp_dir = TempDir::new("test_insert_node_empty_display_name_error")
-            .expect("Failed to create temp dir");
-        let path = temp_dir
-            .path()
-            .join("registry.yaml")
-            .to_str()
-            .expect("Failed to get path")
-            .to_string();
-
-        write_to_file(&vec![], &path);
-
-        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
-
-        let mut node = get_node_1();
-        node.display_name = "".to_string();
-        let result = registry.insert_node(node);
-
-        match result {
-            Ok(_) => panic!("Node display_name is empty. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::EmptyDisplayName)) => {}
-            Err(err) => panic!(
-                "Should have gotten InvalidNodeError::EmptyDisplayName but got {}",
-                err
-            ),
-        }
-    }
-
-    ///
-    /// Verifies that insert_node returns InvalidNodeError::EmptyKey when a node with
-    /// an empty string in its keys is added to the registry.
-    ///
-    #[test]
-    fn test_insert_node_empty_key_error() {
-        let temp_dir =
-            TempDir::new("test_insert_node_empty_key_error").expect("Failed to create temp dir");
-        let path = temp_dir
-            .path()
-            .join("registry.yaml")
-            .to_str()
-            .expect("Failed to get path")
-            .to_string();
-
-        write_to_file(&vec![], &path);
-
-        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
-
-        let mut node = get_node_1();
-        node.keys = vec!["".to_string()];
-        let result = registry.insert_node(node);
-
-        match result {
-            Ok(_) => panic!("Node key is empty. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::EmptyKey)) => {}
-            Err(err) => panic!(
-                "Should have gotten InvalidNodeError::EmptyKey but got {}",
-                err
-            ),
-        }
-    }
-
-    ///
-    /// Verifies that insert_node returns InvalidNodeError::MissingEndpoints when a node with no
-    /// endpoints is added to the registry.
-    ///
-    #[test]
-    fn test_insert_node_missing_endpoints_error() {
-        let temp_dir = TempDir::new("test_insert_node_missing_endpoints_error")
-            .expect("Failed to create temp dir");
-        let path = temp_dir
-            .path()
-            .join("registry.yaml")
-            .to_str()
-            .expect("Failed to get path")
-            .to_string();
-
-        write_to_file(&vec![], &path);
-
-        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
-
-        let mut node = get_node_1();
-        node.endpoints = vec![];
-        let result = registry.insert_node(node);
-
-        match result {
-            Ok(_) => panic!("Node endpoints is empty. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::MissingEndpoints)) => {}
-            Err(err) => panic!(
-                "Should have gotten InvalidNodeError::MissingEndpoints but got {}",
-                err
-            ),
-        }
-    }
-
-    ///
-    /// Verifies that insert_node returns InvalidNodeError::MissingKeys when a node with no
-    /// keys is added to the registry.
-    ///
-    #[test]
-    fn test_insert_node_missing_keys_error() {
-        let temp_dir =
-            TempDir::new("test_insert_node_missing_keys_error").expect("Failed to create temp dir");
-        let path = temp_dir
-            .path()
-            .join("registry.yaml")
-            .to_str()
-            .expect("Failed to get path")
-            .to_string();
-
-        write_to_file(&vec![], &path);
-
-        let registry = LocalYamlRegistry::new(&path).expect("Failed to create LocalYamlRegistry");
-
-        let mut node = get_node_1();
-        node.keys = vec![];
-        let result = registry.insert_node(node);
-
-        match result {
-            Ok(_) => panic!("Node keys is empty. Error should be returned"),
-            Err(RegistryError::InvalidNode(InvalidNodeError::MissingKeys)) => {}
-            Err(err) => panic!(
-                "Should have gotten InvalidNodeError::MissingKeys but got {}",
+                "Should have gotten RegistryError::InvalidStateError but got {}",
                 err
             ),
         }
@@ -1113,7 +1195,7 @@ mod test {
 
         // Verify that the file exists and is empty
         let file = File::open(&path).expect("Failed to open file");
-        let file_contents: Vec<Node> =
+        let file_contents: Vec<YamlNode> =
             serde_yaml::from_reader(file).expect("Failed to deserialize file");
         assert!(file_contents.is_empty());
     }
@@ -1216,7 +1298,11 @@ mod test {
     }
 
     fn write_to_file(data: &[Node], file_path: &str) {
+        let yaml_data: Vec<YamlNode> = data
+            .iter()
+            .map(|node| YamlNode::from(node.clone()))
+            .collect();
         let file = File::create(file_path).expect("Error creating test yaml file.");
-        serde_yaml::to_writer(file, data).expect("Error writing nodes to file.");
+        serde_yaml::to_writer(file, &yaml_data).expect("Error writing nodes to file.");
     }
 }
